package cohy.correlator;
import 	 sep.operator.*;
import   jam.vector.*;
import   juice.vector.*;
import	 rsf.vector.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the correlation between an input data set and a signal data set.
 * The correlation is based on the concept of a supertrace where the input 
 * and signal data are unravelled into two traces. The two traces are then 
 * correlated. The correlation is 1 if and only if the two data sets are 
 * identical. 
 * 
 * The input and signal data can be 1-D, 2-D, or 3-D but have to agree in 
 * shape (probably should agree in space but that is not checked).
 * Output is a RealVec.
 * In the image function: input data reference can be identical to the 
 * constructor's signal reference.
 * @see: TraceCorrelator 
 */
public class Correlator extends SepOperator {
  Rsf     signal;

  /**
   * Input space equals the signal space. Output space defaults to RealSpc.
   */
  public Correlator(Rsf signal) {
    super(signal.getSpace(), RealSpc.getSpace());
    this.signal = signal;
  }
  protected Correlator(Correlator src) {
    super(src);
    this.signal   = (Rsf) src.signal.copy();
  }
  public jam.JamObject copy() { return new Correlator(this);}
  /**
   * I should probably call this createRange(). Defaults to RealSpc.
   */

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  =  this.signal.getDimension();
    if (dim != ((Rsf) iput).getDimension()) 
      System.err.println("Error: dimensional mismatch");
    switch (dim) {
    case 1: apply1d(add, iput, oput); break;
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimension not implemented yet."); 
      break;
    }
  }

  void apply1d(boolean add, Vector iput, Vector oput) {
    float[] data = ((Rsf1)   iput).d;
    float[] sgnl = ((Rsf1) signal).d;
    ((RealVec) oput).setValue(Correlation.correlate(data, sgnl));
  }
  void apply2d(boolean add, Vector iput, Vector oput) {
    float res  = 0.f; 
    float[][] data = ((Rsf2)   iput).d;
    float[][] sgnl = ((Rsf2) signal).d;
    int n1 = data.length;
    for   (int i1 =    0; i1 < n1; i1++) {
      res += Correlation.correlate(data[i1],sgnl[i1]);
    }
    res /= n1; 
    ((RealVec) oput).setValue(res);
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    float res  = 0.f; 
    float[][][] data = ((Rsf3)   iput).d;
    float[][][] sgnl = ((Rsf3) signal).d;
    int n2 = data.length;
    int n1 = data[0].length;
    for   (int i2 = 0; i2 < n2; i2++) {
    for   (int i1 = 0; i1 < n1; i1++) {
      res += Correlation.correlate(data[i2][i1],sgnl[i2][i1]);
    }}
    res /= (n1 * n2); 
    ((RealVec) oput).setValue(res);
  }
}



