package cohy.correlator;
import 	 sep.operator.*;
import   jam.vector.*;
import   juice.vector.*;
import	 rsf.vector.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the correlation between an input data set and a signal data set.
 * The correlation is based on the concept of a supertrace where the input 
 * and signal data are unravelled into two traces. The two traces are then 
 * correlated. The correlation is 1 if and only if the two data sets are 
 * identical. 
 * 
 * The input and signal data can be 1-D, 2-D, or 3-D but have to agree in 
 * shape (probably should agree in space but that is not checked).
 * Output is a RealVec.
 * In the image function: input data reference can be identical to the 
 * constructor's signal reference.
 * @see: TraceCorrelator 
 */
public class CorrelatorToRsf extends SepOperator {
  Correlator corr;
  /**
   * Domain and Range space equals the signal space. 
   */
  public CorrelatorToRsf(Rsf signal) {
    this(signal, (RsfSpace) signal.getSpace());
  }
  /** 
   * Input space equals the signal space. 
   * The output space describes an Rsf that will be filled by the 
   * correlation coefficients (currently accepts RsfSpace or RealSpc).
   */
  public CorrelatorToRsf(Rsf signal, RsfSpace oputSpace) {
    super(signal.getSpace(), oputSpace);
    this.corr = new Correlator(signal); 
  }
  protected CorrelatorToRsf(CorrelatorToRsf src) {
    super(src);
    this.corr   = (Correlator) src.corr.copy();
  }
  public jam.JamObject copy() { return new CorrelatorToRsf(this);}
  /**
   * I should probably call this createRange(). Defaults to RealSpc.
   */
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    return iputSpace; 
  }

  protected void apply(boolean add, Vector iput, Vector oput) {
    RealVec dummy = new RealVec();
    corr.apply(add,iput,dummy);
    ((Rsf) oput).fill(dummy.getValue());
  }
}
