package   cohy.estDip;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   conv.convi.*;
import   sep.io.ArgumentKeeper;

/**
 * THIS NEEDS A MUCH BETTER DOCUMENTATION. FACTS. 
 * Estimates the coherency of a plane-wave component in a data. 
 * A dip estimation (should be argument?) defines the plane wave.
 * Computes the ability of the plane-wave to "explain" the data. 
 * Returns a Rsf vector of coherency values.<BR> 
 * 2-D: single coherency component.<BR>
 * 3-D: 2 components. First is coherency in x, second in y direction.
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class CohEstimator extends SepOperator{
  PartDiffDots pdif; 
  Vector       dots; 

  /** indices that identify the coherency direction in output array.*/
  public final static int CX = 0; 
  public final static int CY = 1;

  public CohEstimator(RsfSpace iputSpace) {
    this(iputSpace, CohEstimator.getOutputRsfSpace(iputSpace));
  }
  public CohEstimator(RsfSpace iputSpace, RsfSpace oputSpace) {
    super(iputSpace, oputSpace);
    int dim   = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
    this.pdif = new PartDiffDots(iputSpace);
    this.dots = pdif.getRange().newMember();
  }
  protected CohEstimator(CohEstimator src) {
    super(src);
    this.pdif = new PartDiffDots((RsfSpace) src.getDomain()); 
    this.dots = pdif.getRange().newMember(); 
  }
  public jam.JamObject copy() { return new CohEstimator(this);}
  /**
   * I should probably call this createRange(). 
   */
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    return new RsfSpace(iputSpace.getDimension() -1);
  }
  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  = ((Rsf) iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    pdif.image(iput, dots); 
    int[] ixx = {PartDiffDots.XX2}; float xx = ((Rsf)dots).getDatum(ixx);
    int[] ixy = {PartDiffDots.XY2}; float xy = ((Rsf)dots).getDatum(ixy);
    int[] iyy = {PartDiffDots.YY2}; float yy = ((Rsf)dots).getDatum(iyy);

    float coh = (float) Math.sqrt((xy/yy) * (xy/xx)); // coh = (xy/yy)*(xy/xx);
    int[] cx  = {CX}; ((Rsf)oput).setDatum(cx, coh);
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    pdif.image(iput, dots);
    int[] ixx = {PartDiffDots.XX3}; float xx = ((Rsf)dots).getDatum(ixx);
    int[] ixz = {PartDiffDots.XZ3}; float xz = ((Rsf)dots).getDatum(ixz);
    int[] iyy = {PartDiffDots.YY3}; float yy = ((Rsf)dots).getDatum(iyy);
    int[] iyz = {PartDiffDots.YZ3}; float yz = ((Rsf)dots).getDatum(iyz);
    int[] izz = {PartDiffDots.ZZ3}; float zz = ((Rsf)dots).getDatum(izz);

    float cohx = (float) Math.sqrt((xz/zz)*(xz/xx)); //coh = (xz/zz)*(xz/xx);
    float cohy = (float) Math.sqrt((yz/zz)*(yz/yy)); //coh = (yz/zz)*(yz/yy);

    // Some special cases. (exhaustive?) 
    // take the limit using l'Hospital rule
    if (xx == 0.f && zz != 0.f || xx != 0.f && zz == 0.f) cohx = 1.f;
    if (yy == 0.f && zz != 0.f || yy != 0.f && zz == 0.f) cohy = 1.f;
    // constant function everywhere coherency is 1 everywhere (true?) 
    if (xx == 0.f && yy == 0.f && zz == 0.f) {
      cohx = 1.f;
      cohy = 1.f;
    }
    int[] cx = {CX}; ((Rsf)oput).setDatum(cx, cohx);
    int[] cy = {CY}; ((Rsf)oput).setDatum(cy, cohy);
  }
}


