package   cohy.estPef;

import  sep.io.ArgumentKeeper;
import 	sep.operator.*;
import 	rsf.vector.*;
import 	jam.vector.*;
import  pef.pef.*;
import conv.convi.*;

/**
 * Computes the Norm of the three two-dimensional PE filter output. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class XPefLocalNorm extends SepLinearOperator{
static int counter = 0; 
  RsfSpace pspc = new RsfSpace(4,4,4);
  RsfSpace xspc = new RsfSpace(4,2,1);
  RsfSpace yspc = new RsfSpace(4,1,2);
  RsfSpace zspc = new RsfSpace(1,4,2);
  int iter      = 6; 

  public XPefLocalNorm(RsfSpace iputSpace, RsfSpace oputSpace, RsfSpace pspc) {
    super(iputSpace, oputSpace); 
    this.pspc = pspc;
  }
  protected XPefLocalNorm(XPefLocalNorm src) {
    super(src);
    this.pspc = src.pspc; 
  }
  public jam.JamObject copy() { return new XPefLocalNorm(this);}

  protected void apply(boolean add, Vector iput, Vector oput) {
    RsfSpace ispc = (RsfSpace) this.getDomain();
    RsfSpace ospc = (RsfSpace) this.getRange();
    Rsf      dumb = (Rsf) ospc.newMember();
    Pef pefi; Rsf filt; Rsf cube; Icai conv; // Icaf maybe more elegant?

//    oput.zero();
    pefi = new Pef(ispc, xspc, iter);
    filt = (Rsf) pefi.getRange().newMember();
    pefi.image(iput, filt);
    cube = this.getCubeFilter(filt);
    conv =                                         
      new IcaiFactory(cube).getIcaiFromInputSpace((RsfSpace) this.getDomain());
    conv.image(iput, dumb);
//dumb.write("xcomp.H");
    rsf.operator.ElwSqr.elwSqr(dumb);
    oput.add(dumb);

    pefi = new Pef(ispc, yspc, iter);
    filt = (Rsf) pefi.getRange().newMember();
    pefi.image(iput, filt);
    cube = this.getCubeFilter(filt);
    conv =                                         
      new IcaiFactory(cube).getIcaiFromInputSpace((RsfSpace) this.getDomain());
    conv.image(iput, dumb);
//dumb.write("ycomp.H");
    rsf.operator.ElwSqr.elwSqr(dumb);
    oput.add(dumb);

    pefi = new Pef(ispc, zspc, iter);
    filt = (Rsf) pefi.getRange().newMember();
    pefi.image(iput, filt);
    cube = this.getCubeFilter(filt);
    conv =                                         
      new IcaiFactory(cube).getIcaiFromInputSpace((RsfSpace) this.getDomain());
    conv.image(iput, dumb);
//dumb.write("zcomp.H");
    rsf.operator.ElwSqr.elwSqr(dumb);
    oput.add(dumb);
//((Rsf) oput).write("oput.H");

    rsf.operator.ElwSqrt.elwSqrt((Rsf) oput);
//((Rsf) oput).write("oput2.H");
//counter++;
//if (counter == 36) System.exit(0);
  }

  Rsf getCubeFilter(Rsf filt) {
    Rsf cube = RsfFactory.newRsf(pspc); 
    float[][][] fdat = ((Rsf3) filt).d; 
    float[][][] cdat = ((Rsf3) cube).d; 
    int n2 = fdat      .length; int n2h = 0;        
    int n1 = fdat[0]   .length; int n1h = (n1-1)/2; 
    int n0 = fdat[0][0].length; int n0h = (n0-1)/2; 
    int d2h = 0                        - n2h; 
    int d1h = (cdat[0]   .length -1)/2 - n1h;
    int d0h = (cdat[0][0].length -1)/2 - n0h;
    for (    int i2=0; i2<n2; i2++) 
      for (  int i1=0; i1<n1; i1++) 
	for (int i0=0; i0<n0; i0++) 
	  cdat[d2h+i2][d1h+i1][d0h+i0] = fdat[i2][i1][i0];
    return cube; 
  }

  public static void main(String[] args) {
    Rsf iput               = RsfFactory.newRsf(System.in);
    RsfSpace pefSpace      = new RsfSpace(4,4,4);
    IcaiPefFactory convFac = new IcaiPefFactory(pefSpace, 10); 
    IcaiPef        conv    = (IcaiPef)convFac.getOpWithDomain(iput.getSpace());
    Rsf            oput    = (Rsf)    conv.getRange().newMember();

    XPefLocalNorm  op = new XPefLocalNorm((RsfSpace) conv.getDomain(),
                                          (RsfSpace) conv.getRange(), 
					  (RsfSpace) pefSpace);
    op.image(iput, oput); 

    oput.write(System.out);
  }
}





