package  cohy.gradient;

import 	  jam.vector.*;
import 	  jam.operator.*;
import 	  rsf.vector.*;
import    rsf.operator.*;
import    sep.io.*;
import    conv.convi.*;

public class Gradient { 
  /**
   * Gradient [weights=1.f,1.f,1.f] <in.H > out.H <BR><BR>
   * @param weights = 1.f,1.f,1.f weights each partial derivative 
   *        by the weight. Specified in order (x,y,z).
   *        (Better set weights individually by static variables)
   */
  public static void main(String[] args) { 
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    float[] weights    = cmd.getFloatArray("weights","1.f,1.f,1.f");

    Rsf iput = RsfFactory.newRsf(System.in); 

    Rsf dx = PartDiff3d.getDerivativeDx();
    Rsf dy = PartDiff3d.getDerivativeDy();
    Rsf dz = PartDiff3d.getDerivativeDz();

    InternalConvolution myInternalConvolution =
      new InternalConvolution( InternalConvolution.kInput , 
			       (RsfSpace) iput.getSpace(),
			       InternalConvolution.kFilter,
			       (RsfSpace) dx.getSpace());

    /*
    // This was nice but wasteful: my machine ran out of memory. 
    Rsf dxIput = (Rsf) myInternalConvolution.imageBinary(iput, dx);
    Rsf dyIput = (Rsf) myInternalConvolution.imageBinary(iput, dy);
    Rsf dzIput = (Rsf) myInternalConvolution.imageBinary(iput, dz);

    Rsf grad = (Rsf) dxIput.getSpace().newMember();
    dxIput.mul(dxIput); grad.add(dxIput);
    dyIput.mul(dyIput); grad.add(dyIput);
    dzIput.mul(dzIput); grad.add(dzIput);
    */

    Rsf dIput; 
    Rsf grad = (Rsf) myInternalConvolution.getRange().newMember();
    dIput = (Rsf) myInternalConvolution.imageBinary(iput, dx);
    dIput.scale(weights[0]); dIput.mul(dIput); grad.add(dIput);
    dIput = (Rsf) myInternalConvolution.imageBinary(iput, dy);
    dIput.scale(weights[1]); dIput.mul(dIput); grad.add(dIput);
    dIput = (Rsf) myInternalConvolution.imageBinary(iput, dz);
    dIput.scale(weights[2]); dIput.mul(dIput); grad.add(dIput);

    ElwSqrt.elwSqrt(grad, grad);

    grad.write(System.out); 
  }



}
