package cohy.patch3dF;

import   jam.vector.*;
import   rsfF.vector.*;
import   rsfF.solver.*;
import  conv.conviF.*;
import sep.io.ArgumentKeeper;
import juice.solver.*;
import  cohy.patchF.*;
import  java.util.Enumeration;

public class NonStationaryPefFilter {
  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);
    // you may have to move the data to that directory
    // String dataDir = "/scrsa1/matt/";
    String dataDir = "../Data/";
    String  inRsf  = argKeeper.getString("in" , dataDir + "gulfAntiRaw.HH");
    String outRsf  = argKeeper.getString("out", dataDir + "NSPEFout.H");
    float  overlap = argKeeper.getFloat ("overlap", "0.5");
    int    niter   = argKeeper.getInt   ("niter"  , "20");
    float  tol     = argKeeper.getFloat ("tol"    , "1e-10");

    Rsf data = new Rsf(inRsf);
    Rsf out  = (Rsf) data.getSpace().newMember();
    Rsf quiltWeight = (Rsf) out.getSpace().newMember();
    out.zero();         // this may be redundant in Java
    quiltWeight.zero();

    int[] patchSize = { ((RsfSpace) data.getSpace()).getNSamples(0),
			10, 10 };  // full version
    //    int[] patchSize = {  4, 10, 10 };  // windowed version
    int[] filtSize  = {  3,  3,  2 };
    int[] filtOne   = {  1,  1,  0 };
    Rsf   pefKnown    = getPefKnown   (filtSize, filtOne);
    Mask  pefFreeMask = getPefFreeMask(filtSize, filtOne);
    Rsf   filter      = (Rsf) pefKnown.getSpace().newMember();
    // set # iterations to roughly 2 * (# unkowns) for CG
    niter = 2 * filtSize[0]*filtSize[1]*filtSize[2];

    int[] patchOutputSize = new int[patchSize.length];
    for (int i = 0; i < patchSize.length; i++)
      patchOutputSize[i] = patchSize[i] - filtSize[i] + 1; // internal conv.
    // patchWeight     is for the OUTPUT space of the internal convolution
    Rsf patchWeight     = getTentWeight3D(patchOutputSize);
    // fullPatchWeight is for the INPUT  space of the internal convolution
    Rsf fullPatchWeight =
      (Rsf) new Patch(new RsfSpace(patchSize),
		      (RsfSpace) patchWeight.getSpace(),
		      filtOne, false).image(patchWeight);

    Patchwork pw = new Patchwork((RsfSpace) data.getSpace(),
				 patchSize,
				 overlap);

    int numPatches = pw.size();
    int i = 1;
    for (Enumeration e = pw.getPatches(); e.hasMoreElements(); i++) {
      System.out.println("Doing patch #" + i + " of " + numPatches);
      Patch cut = (Patch) e.nextElement();
      Vector dataPatch = cut.image(data);
      filter.zero();

      MissingFilter.getMissingFilter((Rsf) dataPatch, pefKnown,
				     pefFreeMask, filter, tol, niter);
      filter.add(pefKnown);

      InternalConvolution
	conv = new InternalConvolution(InternalConvolution.kInput ,
				       (RsfSpace) dataPatch.getSpace(),
				       InternalConvolution.kFilter,
				       (RsfSpace) filter   .getSpace());
      Vector filtered = conv.imageBinary(cut.image(data), filter);
      multiply(patchWeight, (Rsf) filtered, (Rsf) filtered);
      
      Patch  pad      = new Patch((RsfSpace) cut     .getRange(),
				  (RsfSpace) filtered.getSpace(),
				  filtOne, false);
      cut.adjoint().addImage(pad.image(filtered), out);
      Rsf pWeight = (Rsf) pad.getRange().newMember();
      pWeight.setData(fullPatchWeight.d);
      cut.adjoint().addImage(pWeight, quiltWeight);
    }

    divide(out, quiltWeight, out);

    filter.write("filter.H");
    //    fullPatchWeight.write("weight.H");
    //    quiltWeight.write(dataDir + "quilt.H");
    out.write(outRsf);
  }

  static Rsf  getPefKnown   (int[] pefSize, int[] filtOne) {
    Rsf pef = new Rsf(pefSize);
    pef.zero();
    pef.setDatum(filtOne, 1F);
    return pef;
  }

  static Mask getPefFreeMask(int[] pefSize, int[] filtOne) {
    Rsf mask = new Rsf(pefSize);
    mask.fill(1F);
    int n = mask.i(filtOne);
    // This loop depends on the structure of rsfF.vector.Rsf and
    // would be more complicated with rsfA.vector.Rsf
    for (int i = 0; i <= n; i++)
      mask.d[i] = 0F;
    return new SepMask(mask);
  }

  static Rsf getTentWeight3D(int[] spaceSize) {
    Rsf tent = new Rsf(spaceSize);
    int[] i = new int[3];
    int n2 = spaceSize[2] - 1;
    int n1 = spaceSize[1] - 1;
    int n0 = spaceSize[0] - 1;

    for     (i[2] = 0; i[2] <= n2; i[2]++) {
      int     v2 = n2 - Math.abs(2*i[2] - n2) + 2;
      for   (i[1] = 0; i[1] <= n1; i[1]++) {
	int   v1 = n1 - Math.abs(2*i[1] - n1) + 2;
	for (i[0] = 0; i[0] <= n0; i[0]++) {
	  int v0 = n0 - Math.abs(2*i[0] - n0) + 2;
	  tent.d[tent.i(i)] = ((((long) v2)*v1)*v0) / (8F);
	}}}
    return tent;
  }

  static void multiply(Rsf r1, Rsf r2, Rsf prod) {
    for (int i = 0; i < r1.d.length; i++)
      prod.d[i] = r1.d[i] * r2.d[i];
  }

  static void divide  (Rsf r1, Rsf r2, Rsf div) {
    for (int i = 0; i < r1.d.length; i++)
      if (r2.d[i] != 0)
	div.d[i] = r1.d[i] / r2.d[i];
  }

  // this should probably be replace with the Laplacian in rsf.util.Laplacian
  // Rsf mylap = rsf.util.Laplacian.newRsf(3);
  static Rsf getLaplacian() {
    int[] dims = { 3, 3, 3 };
    Rsf lap = new Rsf(dims);
    lap.fill(0F);
    int[] i = new int[3];
    i[0] = 0; i[1] = 1; i[2] = 1; lap.setDatum(i, -1F);
    i[0] = 2; i[1] = 1; i[2] = 1; lap.setDatum(i, -1F);
    i[0] = 1; i[1] = 0; i[2] = 1; lap.setDatum(i, -1F);
    i[0] = 1; i[1] = 2; i[2] = 1; lap.setDatum(i, -1F);
    i[0] = 1; i[1] = 1; i[2] = 0; lap.setDatum(i, -1F);
    i[0] = 1; i[1] = 1; i[2] = 2; lap.setDatum(i, -1F);
    i[0] = 1; i[1] = 1; i[2] = 1; lap.setDatum(i,  8F);
    return lap;
  }
}
